@@ -10,6 +10,4 @@ void set_beta(DATA **d, int sim, int n_vars, METHOD method);
 void setup_beta(DATA **d, int n_vars, int n_sim);
 void print_sim(void);
 void free_simulations(void);
-#ifndef SIM_DOUBLE
 float ***get_msim(void);
-#endif
