@@ -1,30 +1,3 @@
-/*
-    Gstat, a program for geostatistical modelling, prediction and simulation
-    Copyright 1992, 2011 (C) Edzer Pebesma
-
-    Edzer Pebesma, edzer.pebesma@uni-muenster.de
-	Institute for Geoinformatics (ifgi), University of Münster 
-	Weseler Straße 253, 48151 Münster, Germany. Phone: +49 251 
-	8333081, Fax: +49 251 8339763  http://ifgi.uni-muenster.de 
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version. As a special exception, linking 
-    this program with the Qt library is permitted.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-    (read also the files COPYING and Copyright)
-*/
-
 /*
  * iterated mivque (reml) estimate of covariance components, following
  * Christensen's 1994 derivations Math.Geol. 25 (5), 541-558, and
@@ -51,6 +24,7 @@
 #include "vario.h"
 #include "glvars.h"
 #include "select.h"
+#include "mtrx.h"
 #include "lm.h" /* get_X, get_y */
 #include "reml.h"
 
@@ -61,6 +35,7 @@ static MAT *calc_VinvIminAw(MAT *Vw, MAT *X, MAT *VinvIminAw, int calc_Aw);
 static void calc_rhs_Tr_m(int n_models, MAT **Vk,MAT *VinvIminAw, 
 	VEC *Y, VEC *rhs, MAT *Tr_m);
 static double calc_ll(MAT *Vw, MAT *X, VEC *y, int n);
+static MAT *XVXt_mlt(MAT *X, MAT *V, MAT *out);
 
 static MAT *IminAw = MNULL;
 
@@ -92,7 +67,7 @@ VARIOGRAM *reml_sills(DATA *data, VARIOGRAM *vp) {
 	dzero2 = gl_zero * gl_zero;
 	for (i = 0; i < data->n_list; i++) {
 		for (j = 0; j < vp->n_models; j++) /* fill diagonals */
-			Vk[j]->me[i][i] = Covariance(vp->part[j], 0.0, 0.0, 0.0);
+			ME(Vk[j], i, i) = Covariance(vp->part[j], 0.0, 0.0, 0.0);
 		for (j = 0; j < i; j++) { /* off-diagonal elements: */
 			dpa = data->list[i];
 			dpb = data->list[j];
@@ -112,8 +87,7 @@ VARIOGRAM *reml_sills(DATA *data, VARIOGRAM *vp) {
 					dz = (dz >= 0 ? gl_zero : -gl_zero);
 			}
 			for (k = 0; k < vp->n_models; k++)
-				Vk[k]->me[i][j] = Vk[k]->me[j][i] = 
-						Covariance(vp->part[k], dx, dy, dz);
+				ME(Vk[k], i, j) = ME(Vk[k], j, i) = Covariance(vp->part[k], dx, dy, dz);
 		}
 	}
 	if (reml(Y, X, Vk, vp->n_models, gl_iter, gl_fit_limit, init))
@@ -158,12 +132,12 @@ static int reml(VEC *Y, MAT *X, MAT **Vk, int n_k, int max_iter,
 		VinvIminAw = calc_VinvIminAw(Vw, X, VinvIminAw, n_iter == 1);
 		calc_rhs_Tr_m(n_k, Vk, VinvIminAw, Y, rhs, Tr_m);
 		/* Tr_m * teta = Rhs; symmetric, solve for teta: */
-		LDLfactor(Tr_m, &info);
+		CHfactor(Tr_m, PNULL, &info);
 		if (info != 0) {
 			pr_warning("singular matrix in reml");
 			return(0);
 		}
-		LDLsolve(Tr_m, rhs, teta);
+		CHsolve1(Tr_m, rhs, teta, PNULL);
 		if (DEBUG_VGMFIT) {
 			printlog("teta_%d [", n_iter);
 			for (i = 0; i < teta->dim; i++)
@@ -188,7 +162,7 @@ static int reml(VEC *Y, MAT *X, MAT **Vk, int n_k, int max_iter,
 			ms_mltadd(Vw, Vk[i], teta->ve[i], Vw); /* Vw = Sum_i teta[i]*V[i] */
 		VinvIminAw = calc_VinvIminAw(Vw, X, VinvIminAw, 0);
 		calc_rhs_Tr_m(n_k, Vk, VinvIminAw, Y, rhs, Tr_m);
-		m_inverse(Tr_m, Tr_m, &info);
+		m_inverse(Tr_m, &info);
 		sm_mlt(2.0, Tr_m, Tr_m); /* Var(YAY)=2tr(AVAV) */
 		printlog("Lower bound of parameter covariance matrix:\n");
 		m_logoutput(Tr_m);
@@ -211,7 +185,7 @@ static MAT *calc_VinvIminAw(MAT *Vw, MAT *X, MAT *VinvIminAw, int calc_Aw) {
  * calc (I-Aw) only once and keep this constant during iteration.
  */
  	MAT *tmp = MNULL, *V = MNULL;
- 	VEC *b = VNULL, *rhs = VNULL;
+ 	/* VEC *b = VNULL, *rhs = VNULL; */
  	int i, j, info;
 
 	if (X->m != Vw->n || VinvIminAw->m != X->m)
@@ -221,7 +195,7 @@ static MAT *calc_VinvIminAw(MAT *Vw, MAT *X, MAT *VinvIminAw, int calc_Aw) {
 		IminAw = m_resize(IminAw, X->m, X->m);
 		tmp = m_resize(tmp, X->n, X->n);
 		tmp = mtrm_mlt(X, X, tmp); /* X'X */
-		m_inverse(tmp, tmp, &info); /* (X'X)-1 */
+		m_inverse(tmp, &info); /* (X'X)-1 */
 		if (info != 0)
 			pr_warning("singular matrix in calc_VinvIminAw");
 		/* X(X'X)-1 -> X(X'X)-1 X') */
@@ -229,26 +203,17 @@ static MAT *calc_VinvIminAw(MAT *Vw, MAT *X, MAT *VinvIminAw, int calc_Aw) {
 		for (i = 0; i < IminAw->m; i++) /* I - Aw */
 			for (j = 0; j <= i; j++)
 				if (i == j)
-					IminAw->me[i][j] = 1.0 - IminAw->me[i][j];
+					ME(IminAw, i, j) = 1.0 - ME(IminAw, i, j);
 				else
-					IminAw->me[i][j] = IminAw->me[j][i] = -IminAw->me[i][j];
+					ME(IminAw, i, j) = ME(IminAw, j, i) = -(ME(IminAw, i, j));
 	}
 
 	V = m_copy(Vw, V);
-	LDLfactor(V, &info);
+	CHfactor(V, PNULL, &info);
 	if (info != 0)
 		pr_warning("singular V matrix in calc_VinvIminAw");
 
-	rhs = v_resize(rhs, X->m);
-	b = v_resize(b, X->m);
-
-	for (i = 0; i < X->m; i++) { /* solve Vw X = (I-A) for X -> V-1(I-A) */
-		rhs = get_col(IminAw, i, rhs);
-		LDLsolve(V, rhs, b);
-		set_col(VinvIminAw, i, b);
-	}
-	v_free(rhs);
-	v_free(b);
+	CHsolve(V, IminAw, VinvIminAw, PNULL);
 	m_free(V);
 
 	if (tmp) 
@@ -268,11 +233,11 @@ static void calc_rhs_Tr_m(int n_models, MAT **Vk,MAT *VinvIminAw,
 	for (j = 0; j < n_models; j++) {
 		Pr[j] = m_mlt(Vk[j], VinvIminAw, MNULL);
 		Tmp = m_mlt(Pr[j], Pr[j], Tmp);
-		Tr_m->me[j][j] = trace_matrix(Tmp); /* diagonal */
+		ME(Tr_m, j, j) = trace_matrix(Tmp); /* diagonal */
 		/* using Tr(A B) == Tr(B A) */
 		for (k = 0; k < j; k++) { /* we did Pr[k] and Pr[j], so */
 			Tmp = m_mlt(Pr[j], Pr[k], Tmp); /* off-diagonal */
-			Tr_m->me[j][k] = Tr_m->me[k][j] = trace_matrix(Tmp);
+			ME(Tr_m, j, k) = ME(Tr_m, k, j) = trace_matrix(Tmp);
 		}
 		v_tmp = vm_mlt(Vk[j], v_tmp2, v_tmp); /* Vw-1(I-Aw)Y */
 		rhs->ve[j] = in_prod(v_tmp2, v_tmp);
@@ -301,14 +266,14 @@ static double calc_ll(MAT *Vw, MAT *X, VEC *y, int n) {
 	/* |B'(I-A)Vw(I-A)'B|, pretty inefficiently, can 4 x as fast: */
 	/* M1 = m_mlt(IminAw, Vw, M1); M2 = mmtr_mlt(M1, IminAw, M2); */
 	M1 = XVXt_mlt(IminAw, Vw, M1);
-	LDLfactor(M1, &info);
+	CHfactor(M1, PNULL, &info);
 	for (i = 0, ldet = 0.0; i < M1->m; i++) {
-		assert(M1->me[i][i] > 0.0);
-		ldet += log(M1->me[i][i]);
+		assert(ME(M1, i, i) > 0.0);
+		ldet += log(ME(M1, i, i));
 	}
 	/* y'B'A'(B'A'Vw A B)-1 A B y */
 	res = mv_mlt(IminAw, y, res); /* the m-n residuals B(I-A)'Y */
-	tmp = LDLsolve(M1, res, tmp);
+	tmp = CHsolve1(M1, res, tmp, PNULL);
 		/* M1 tmp = res -> tmp = M1-1 res */
 	zQz = in_prod(res, tmp);  /* res' M1inv res */
 
@@ -327,7 +292,7 @@ static double trace_matrix(MAT *m) {
 	if (m->m != m->n)
 		ErrMsg(ER_IMPOSVAL, "trace_matrix: non-square matrix");
 	for (i = 0, trace = 0.0; i < m->m; i++)
-		trace += m->me[i][i];
+		trace += ME(m, i, i);
 	return trace;
 }
 
@@ -351,14 +316,14 @@ MAT *XtVX_mlt(MAT *X, MAT *V, MAT *out) {
 	for (i = 0; i < X->n; i++) {
 		for (j = i; j < X->n; j++)
 			for (k = 0; k < X->m; k++)
-				out->me[i][j] += X->me[k][i] * VX->me[k][j];
+				ME(out, i, j) += ME(X, k, i) * ME(VX, k, j);
 		for (j = 0; j <= i; j++) /* symmetry */
-			out->me[i][j] = out->me[j][i];
+			ME(out, i, j) = ME(out, j, i);
 	}
 	return out;
 }
 
-MAT *XVXt_mlt(MAT *X, MAT *V, MAT *out) {
+static MAT *XVXt_mlt(MAT *X, MAT *V, MAT *out) {
 /* for a symmetric matrix V, return X V X' */
 	static MAT *VXt = MNULL;
 	int i, j, k;
@@ -378,9 +343,9 @@ MAT *XVXt_mlt(MAT *X, MAT *V, MAT *out) {
 	for (i = 0; i < X->m; i++) {
 		for (j = i; j < X->m; j++)
 			for (k = 0; k < X->n; k++)
-				out->me[i][j] += X->me[i][k] * VXt->me[k][j];
+				ME(out, i, j) += ME(X, i, k) * ME(VXt, k, j);
 		for (j = 0; j <= i; j++) /* symmetry */
-			out->me[i][j] = out->me[j][i];
+			ME(out, i, j) = ME(out, j, i);
 	}
 	return out;
 }
@@ -400,9 +365,9 @@ MAT *XdXt_mlt(MAT *X, VEC *d, MAT *out) {
 	for (i = 0; i < X->m; i++) {
 		for (j = i; j < X->m; j++)
 			for (k = 0; k < X->n; k++)
-				out->me[i][j] += X->me[i][k] * X->me[j][k] * d->ve[k];
+				ME(out, i, j) += ME(X, i, k) * ME(X, j, k) * d->ve[k];
 		for (j = 0; j <= i; j++) /* symmetry */
-			out->me[i][j] = out->me[j][i];
+			ME(out, i, j) = ME(out, j, i);
 	}
 	return out;
 }
@@ -422,9 +387,9 @@ MAT *XtdX_mlt(MAT *X, VEC *d, MAT *out) {
 	for (i = 0; i < X->n; i++) {
 		for (j = i; j < X->n; j++)
 			for (k = 0; k < X->m; k++)
-				out->me[i][j] += X->me[k][i] * X->me[k][j] * d->ve[k];
+				ME(out, i, j) += ME(X, k, i) * ME(X, k, j) * d->ve[k];
 		for (j = 0; j <= i; j++) /* symmetry */
-			out->me[i][j] = out->me[j][i];
+			ME(out, i, j) = ME(out, j, i);
 	}
 	return out;
 }
